/*
 *	@title		Control Functions
 *	@context  	Any
 *  @author		M.Vasiladiotis <michail.vasiladiotis@epfl.ch> & C.Rod
 *	@file		control_functions.c
 */

#include "DSP28x_Project.h"  				// Headers for C28346
#include "../My_functions/config.h"			// Application-specific hardware configuration parameters
#include "control_functions.h"				// Corresponding header file
#include "transformations.h"				// Transformations of three-phase magnitudes

#include <cmath>							// Standard math library

/* ------------------------------------- NOTE -------------------------------------------------------
 * The following functions have been written to act as application-independent. Some argument
 * structures include constants related to each function, which can be pre-calculated in the
 * UserInit() function so as to avoid excessive computations. These are commented next to the
 * respective structure definitions. Note that these constants can be also defined and pre-compiled
 * on a separate file, such as config.h
---------------------------------------------------------------------------------------------------*/


/**
 * Function for line current vector control of voltage source converters in rotating reference frame,
 * featuring decoupled axis capability. This implementation is based on the Tustin method (bilinear) for
 * transformation from continuous (s) to discrete (z) domain: s ~ 2/Ts*(z-1)/(z+1)
 */
void VectorControlPI(SpacePhasor *references, const SpacePhasor *input, SpacePhasor *feedforward, SpacePhasor *output, DecoupledControl *parameters)
{
   float32 ErrorD, ErrorQ ;

   ErrorD = (*references).real - (*input).real;
   ErrorQ = (*references).imaginary - (*input).imaginary;

   (*parameters).uD = (*parameters).uD + ErrorD*((*parameters).Tustin1+(*parameters).Tustin2) + (*parameters).ErrorDprev*((*parameters).Tustin1-(*parameters).Tustin2);
   (*parameters).uQ = (*parameters).uQ + ErrorQ*((*parameters).Tustin1+(*parameters).Tustin2) + (*parameters).ErrorQprev*((*parameters).Tustin1-(*parameters).Tustin2);

   (*output).real = -(*parameters).uD + (*feedforward).real + (*parameters).omegaL*(*input).imaginary;
   (*output).imaginary = -(*parameters).uQ + (*feedforward).imaginary - (*parameters).omegaL*(*input).real;

   /* Storing values for next function call */
   (*parameters).ErrorDprev = ErrorD;
   (*parameters).ErrorQprev = ErrorQ;
}


/* Generic function for PID controller with feed-forward compensation, featuring anti-reset wind-up. Implementation proposed by Prof. Bhler */
float32 ScalarControlPID(float32 Input, float32 Reference, PIDreg *parameters)
{
	float32 Error, OutputREG, Output;

	Error = Reference - Input;
	OutputREG = (*parameters).xREG + ((*parameters).Kp + (*parameters).Ki + (*parameters).Kd)*Error - (*parameters).Kd*(*parameters).ErrorPREV + (*parameters).FeedForward;

    /* Limitations */
    if (OutputREG > (*parameters).UpperLimit) {Output = (*parameters).UpperLimit;}
    else if (OutputREG < (*parameters).LowerLimit) {Output = (*parameters).LowerLimit;}
    else {Output = OutputREG;}

    /* Anti Reset Wind-up */
    (*parameters).xREG = (*parameters).xREG + (*parameters).Ki*(Error-(OutputREG-Output)/((*parameters).Kp + (*parameters).Ki + (*parameters).Kd));

	(*parameters).ErrorPREV = Error;

    return Output;
}


/* Phase-Locked Loop function based on dq transformation (loop filter on the q-axis) */
float32 PLL(SpacePhasor *rotating, PLLparameters *parameters)
{
    float32  omegaPLL, x1, x2;

    x1 = (-(*parameters).kB+(*parameters).aB*(*parameters).Ts/2.0)*(*parameters).IMprev;
	x2 = ((*parameters).kB+(*parameters).aB*(*parameters).Ts/2.0)*(*rotating).imaginary;

	(*parameters).DomegaPLL = (*parameters).DomegaPLL + x1 + x2 ;
	omegaPLL = (*parameters).DomegaPLL + (*parameters).omega ;

	(*parameters).THETA = (*parameters).THETA + (*parameters).Ts * ((*parameters).omegaPLLprev + omegaPLL) / 2;

    /* Avoiding integral saturation */
	if ((*parameters).THETA > 2.0 * PI) {(*parameters).THETA = (*parameters).THETA - 2.0 * PI;}
	if ((*parameters).THETA < -2.0 * PI) {(*parameters).THETA = (*parameters).THETA + 2.0 * PI;}

	/* Storing values for next function call */
	(*parameters).IMprev = (*rotating).imaginary;
	(*parameters).omegaPLLprev = omegaPLL;

    return (*parameters).THETA;
}


/* Function for Fictive Axis Emulation in single-phase systems */
float32 FAE(SpacePhasor *voltage, SpacePhasor *current, FAEparameters *parameters, float32 VimFB)
{
	float32 vFAE;

    /* FAE current output */
	vFAE = (*voltage).imaginary - VimFB;
    (*current).imaginary = (vFAE + (*parameters).vFAEprev + (*parameters).FAEconstant1 * (*current).imaginary) / (*parameters).FAEconstant2  ;

    (*parameters).vFAEprev = vFAE;
    return (*current).imaginary;
}


/* Function for Second-Order Generalized Integrator */
void SOGI(float32 *Input, SpacePhasor *fixed, SOGIparameters *parameters)
{
    // SOGI
    (*fixed).real = ((*parameters).B0*(*Input) + ((*parameters).A1*(*parameters).SOGIstate1 + ((*parameters).A2-1)*(*parameters).SOGIstate2));
    (*fixed).imaginary = (*parameters).wSOGI*(*parameters).SOGIstate1;

    (*parameters).SOGIstate2 = (*parameters).SOGIstate1;
    (*parameters).SOGIstate1 = (*parameters).B0*(*fixed).real + ((*parameters).A2*(*parameters).SOGIstate2 + (*parameters).A1*(*parameters).SOGIstate1);
}


/* Function for a first-order low-pass filter */
float32 LowPassFilter(float32 FilterInput, LPFparameters *parameters)
{
	return (*parameters).FilterOutput = (1 - (*parameters).FilterConstant)*(*parameters).FilterOutput + (*parameters).FilterConstant*FilterInput;
}


/* Function for ramped increase/decrease of a reference value */
float32 RampFunction (float32 ValueINIT, float32 ValueFINAL, Ramp *parameters)
{
	if (abs((*parameters).OutRamp) >= abs(ValueFINAL))
	{
		(*parameters).OutRamp = ValueFINAL;
		(*parameters).time = 0;
	}
	else
	{	(*parameters).OutRamp = ValueINIT + (*parameters).slope * (*parameters).time;
		(*parameters).time += (*parameters).Tsample;
	}

	return (*parameters).OutRamp;
}

