#ifndef CONTROL_FUNCTIONS_H_
#define CONTROL_FUNCTIONS_H_

#include "transformations.h"				// Transformations of three-phase magnitudes

/* ------------------------------------- NOTE -------------------------------------------------------
 * The following functions have been written to act as application-independent. Some argument
 * structures include constants related to each function, which can be pre-calculated in the
 * UserInit() function so as to avoid excessive computations. These are commented next to the
 * respective structure definitions. Note that these constants can be also defined and pre-compiled
 * on a separate file, such as config.h
---------------------------------------------------------------------------------------------------*/


/*-------------------------Data structures utilized for control functions----------------------
 * --------------------------------------------------------------------------------------------*/

/* Parameters for PI-based current vector controller in rotating reference frame with decoupling axis
 * feature. Tn and Ti correspond to a typical transfer function of a scalar regulator in Laplace domain
 * as in G(s) = (1+sTn)/sTi, Tsample is the discretization sampling time and period of the interrupt where
 * the function is executed.
 */
typedef struct{
	float32 Gain;
	float32 Tn;
	float32 Ti;
	float32 Tustin1;		// Tsample/(2*Ti)
	float32 Tustin2;		// Tn/Ti
	float32 omegaL;			// omega*L
	float32 uD;
	float32 uQ;
	float32 ErrorDprev;
	float32 ErrorQprev;		// uD, uQ, ErrorDprev, ErrorQprev should just be initialized to '0.0'
} DecoupledControl;

/* Parameters for scalar PID controller with feed-forward compensation*/
typedef struct{
	float32 Kp;
	float32 Ki;
	float32 Kd;
	float32 UpperLimit;
	float32 LowerLimit;
	float32 FeedForward;
	float32 xREG;
	float32 ErrorPREV;		// xREG, ErrorPREV should just be initialized to '0.0'
} PIDreg;

/**
 *  Parameters for the dq-based Phase-Locked Loop. Parameter omega refers to the desired angular frequency of the
 *  sinusoidal signal, aB and kB are predetermined gains, Ts is the sampling time.
 */
typedef struct{
	float32 THETA;
	float32 Ts;
	float32 aB;
	float32 kB;
	float32 omega;
	float32 DomegaPLL;
	float32 omegaPLLprev;
	float32 IMprev;			// DomegaPLL, omegaPLLprev, IMprev should just be initialized to '0.0'
} PLLparameters;

/**
 *	Parameters for Fictive Axis Emulator. L and R are the line-related parameters (inductance and resistance
 *	respectively), Tsample is the discretization sampling time and period of the interrupt where the function
 *  is executed.
 */
typedef struct{
	float32 FAEconstant1;		//	2*L/Tsample-R;
	float32 FAEconstant2;		// 	2*L/Tsample+R;
	float32 vFAEprev;			// should just be initialized to '0.0'
} FAEparameters;

/**
 * Parameters for second-order generalized integrator, OMEGA_REF is the angular frequency of the sinusoidal
 * signal and Tsample is the discretization sampling time and period of the interrupt where the function
 * is executed.
 */
typedef struct{
	float32 Gain;
	float32 tmp1SOGI;		// 	2*Gain*OMEGA_REF*Tsample
	float32 tmp2SOGI;		// 	pow(OMEGA_REF*Tsample,2)
	float32 A1;				// 	(8-2*tmp2SOGI)/(tmp1SOGI+tmp2SOGI+4)
	float32 A2;				//	(tmp1SOGI-tmp2SOGI-4)/(tmp1SOGI+tmp2SOGI+4);
	float32 B0;				// 	tmp1SOGI/(tmp1SOGI+tmp2SOGI+4)
	float32 wSOGI;			//	2*Tsample*OMEGA_REF;
	float32 SOGIstate1;
	float32 SOGIstate2;		// SOGIstate1, SOGIstate2 should be initialized to '0.0'
} SOGIparameters;


/* Parameters for first-order low-pass filter*/
typedef struct{
	float32 FilterConstant;
	float32	FilterOutput;	// should just be initialized to '0.0'
} LPFparameters;


/* Parameters for ramp function*/
typedef struct{
	float32 OutRamp;
	float32 slope;
	float32 Tsample;
	float32 time;			// time should be initialized to '0.0'
}Ramp;


/*---------------------------------Control Function Prototypes-----------------------------------------------
 * -----------------------------------------------------------------------------------------------------------*/

/**
 * Function for line current vector control of voltage source converters in rotating reference frame,
 * featuring decoupled axis capability. This implementation is based on the Tustin method (bilinear) for
 * transformation from continuous (s) to discrete (z) domain: s ~ 2/Ts*(z-1)/(z+1).
 */
void VectorControlPI(SpacePhasor *references, const SpacePhasor *input, SpacePhasor *feedforward,
		SpacePhasor *output, DecoupledControl *parameters);

/* Function for PI controller with feed-forward compensation, featuring anti-reset wind-up */
float32 ScalarControlPID(float32 Input, float32 Reference, PIDreg *parameters);


/* Phase-Locked Loop function based on dq transformation (loop filter on the q-axis) */
float32 PLL(SpacePhasor *rotating, PLLparameters *parameters);


/* Function for Fictive Axis Emulation in single-phase systems */
float32 FAE(SpacePhasor *voltage, SpacePhasor *current, FAEparameters *parameters, float32 VOLTAGEfeedback);


/* Function for Second-Order Generalized Integrator */
void SOGI(float32 *Input, SpacePhasor *fixed, SOGIparameters *parameters);


/* Function for a first-order low-pass filter */
float32 LowPassFilter(float32 FilterInput, LPFparameters *parameters);


/* Function for ramp increase/decrease of a reference value */
float32 RampFunction (float32 ValueINIT, float32 ValueFINAL, Ramp *parameters);


#endif /*CONTROL_FUNCTIONS_H_*/
