/*
 *	@title		Reference frame transformation and synchronization functions
 *	@context  	Any
 *  @author		M.Vasiladiotis <michail.vasiladiotis@epfl.ch>, C.Rod <christian.rod@epfl.ch> & Louis De Mallac <louis.demallac@epfl.ch>
 *	@file		transformations.c
 */

#include "DSP28x_Project.h"  				// Headers for C28346
#include "transformations.h"			    // Corresponding header file
#include "../My_functions/config.h"			// General user constants and parameters
#include <cmath>

float32 ONE_OVER_SQRT_3=0.577350269;
float32 SQRT_3_OVER_2=0.866025403;

/* ------------------------------------- NOTE --------------------------------------------
 * The three-phase systems are assumed to be symmetric. Therefore, no homopolar component
 * exists and one equation can be omitted for reduction of the computational burden.
 * TO BE UPDATED...? (for asymmetrical networks)
----------------------------------------------------------------------------------------*/

/* Transformation from physical (ABC) to stationary (Alphabeta) reference frame */
void ABC2AlphaBeta(SpacePhasor *fixed, const TimeDomain *physical)
{
	(*fixed).real = (1/3.)*(2*(*physical).A-(*physical).B-(*physical).C);
	(*fixed).imaginary = ONE_OVER_SQRT_3*((*physical).B-(*physical).C);
}


/* Transformation from stationary (AlphaBeta) to physical (ABC) reference frame */
void AlphaBeta2ABC(TimeDomain *physical, const SpacePhasor *fixed)
{
	(*physical).A = (*fixed).real;
	(*physical).B = -0.5*(*fixed).real + SQRT_3_OVER_2*(*fixed).imaginary;
	(*physical).C = -0.5*(*fixed).real - SQRT_3_OVER_2*(*fixed).imaginary;
}


/* Transformation from stationary (AlphaBeta) to rotating (DQ) reference frame */
void AlphaBeta2DQ(SpacePhasor *rotating, const SpacePhasor *fixed, const float32 theta)
{
	float32 cosTheta, sinTheta;

	cosTheta=cos(theta);
	sinTheta=sin(theta);

	(*rotating).real = (*fixed).real*cosTheta + (*fixed).imaginary*sinTheta;
	(*rotating).imaginary = -(*fixed).real*sinTheta + (*fixed).imaginary*cosTheta;
}


/* Transformation from rotating (DQ) to stationary (Alphabeta) reference frame */
void DQ2AlphaBeta(SpacePhasor *fixed, const SpacePhasor *rotating, const float32 theta)
{
	float32 cosTheta, sinTheta;

	cosTheta=cos(theta);
	sinTheta=sin(theta);

	(*fixed).real = (*rotating).real * cosTheta - (*rotating).imaginary * sinTheta;
	(*fixed).imaginary = (*rotating).real * sinTheta + (*rotating).imaginary * cosTheta;
}


/* Transformation from physical (ABC) to rotating (DQ) reference frame */
void ABC2DQ(SpacePhasor *rotating, const TimeDomain *physical, const float32 theta)
{
	float32 alpha, beta, cosTheta, sinTheta;

	cosTheta=cos(theta);
	sinTheta=sin(theta);

	alpha = (1/3.)*(2*(*physical).A-(*physical).B-(*physical).C);
	beta = ONE_OVER_SQRT_3*((*physical).B-(*physical).C);

	(*rotating).real = alpha*cosTheta + beta*sinTheta;
	(*rotating).imaginary = beta*cosTheta - alpha*sinTheta;
}


/* Transformation from rotating (DQ) to physical (ABC) reference frame */
void DQ2ABC(TimeDomain *physical, const SpacePhasor *rotating, const float32 theta)
{
	float32 alpha, beta, cosTheta, sinTheta;

	cosTheta=cos(theta);
	sinTheta=sin(theta);

	alpha = (*rotating).real*cosTheta - (*rotating).imaginary*sinTheta;
	beta = (*rotating).imaginary*cosTheta + (*rotating).real*sinTheta;

	(*physical).A = alpha;
	(*physical).B = -0.5*alpha + SQRT_3_OVER_2*beta;
	(*physical).C = -0.5*alpha - SQRT_3_OVER_2*beta;
}
