#ifndef CORE_H_
#define CORE_H_


/**
 * Enum type for the state machine
 */
typedef enum
{
	FAULT  		= 0,			// The Bbox received an error signal and waits for an acknowledgment from the user to go back to the BLOCKED state
	BLOCKED   	= 1,			// The Bbox is configured and waits for the enable command to activate its outputs
	OPERATING   = 2				// The Bbox is running properly and is ready to receive the disable command or an error signal
} CORE_STATE;


/**
 * Informs the user about "who" asked to enter in error mode
 * This enum is intended to be passed when calling UserError();
 */
typedef enum
{
	HARD = 0,			// Signal FAULT from PERIPH
	SOFT = 1,			// Error from the core software
	USER = 2			// User error from his software
} ERROR_SOURCE;

/**
 * Configures the PIE vector in order that each interrupt
 * maps to the function UnmappedInterruptTrap. Thus inexpected interruptions
 * are catchable and are easy to debug
 */
void SetPieVectTable(void);


/**
 * Does the initialization of the core.
 * The peripherals, watchdog and I/Os are configured in this function. This function
 * is also calling the UserInit() function.
 */
void CoreInit(void);

/**
 * Do all things the core has to do to start the experiment (start interrupts, open outputs,...)
 */
void CoreStart();

/**
 * Do all things the core has to do to stop the experiment (stop interrupts, close outputs,...)
 */
void CoreStop();

/**
 * Do all things the core has to do in background.
 * All tasks not time dependent are done inside this function.
 * The watchdog counter is also reset.
 * @return void
 */
void CoreBackground(void);

/**
 * This function returns the core state.
 */
CORE_STATE GetCoreState(void);

/**
 * This function is the ISR serving the most critical hardware (applicative) errors
 * Its purpose is to prepare proper CPU reboot. This function calls UserError().
 */
void CoreError(ERROR_SOURCE source);

/**
 * This function is the functional opposite of CoreError()..
 * Its purpose is to properly restore the core after a fault.
 */
void CoreRecover(ERROR_SOURCE source);

#endif /*CORE_H_*/
