/*
 *	@title		Discrete implementation of a SOGI using three integrator-discretization of '1/s'
 *	@context  	BoomBox & MMC-Box
 *  @author		O.C.Spro <ocspro@gmail.com>
 *	@file		SOGI_3I.c
 *	@comment	last updated: 31.07.2013
 *				The method is well documentated in the following article:
 *
 *				A New Single-Phase PLL Structure Based on Second Order Generalized Integrator
 *				Mihai Ciobotaru, Remus Teodorescu and Frede Blaabjerg
 * 				Aalborg University, Institute of Energy Technology, DENMARK
 *
 *				Link on ieeexplorer: 10.1109/PESC.2006.1711988
 *				URL: http://ieeexplore.ieee.org/stamp/stamp.jsp?tp=&arnumber=1711988&isnumber=36090
 */

#include "config.h"					// Project configuration containing definitions of SOGI_GAIN, SOGI_CONST and SOGI_OMEGA
#include "SOGI3I.h"					// This file header.

/*
 *	Initialize the SOGI module with default values. SOGI_GAIN, SOGI_CONST and SOGI_OMEGA must be predefined.
 */
void SOGI_init(SOGI_3I_module *module)
{
	module->states[0].z1 = 0;
	module->states[0].z2 = 0;
	module->states[0].z3 = 0;
	module->states[0].output = 0;

	module->states[1].z1 = 0;
	module->states[1].z2 = 0;
	module->states[1].z3 = 0;
	module->states[1].output = 0;

	module->omega = SOGI_OMEGA;
	module->gain = SOGI_GAIN;
	module->constant = SOGI_CONST;
}

/*
 *	Update the SOGi module, takes the measurement value as input.
 */
void SOGI_step(SOGI_3I_module *module, float measurement)
{
	/* FIRST INTEGRATOR BLOCK */

	//Changing values of the states (step z)
	module->states[0].z3 = module->states[0].z2;
	module->states[0].z2 = module->states[0].z1;

	//Calculate value of the first state
	module->states[0].z1 += module->constant * module->omega *(-module->states[1].output + module->gain *(measurement - module->states[0].output));

	/* SECOND INTEGRATOR BLOCK */

	//Changing values of the  states (step z)
	module->states[1].z3 = module->states[1].z2;
	module->states[1].z2 = module->states[1].z1;

	//calculate value of the first state
	module->states[1].z1 += module->constant * module->omega * module->states[0].output;

	//UPDATE OUTPUT FROM BOTH INTEGRATOR BLOCKS
	module->states[0].output = 23*module->states[0].z1 - 16*module->states[0].z2 + 5*module->states[0].z3;
	module->states[1].output = 23*module->states[1].z1 - 16*module->states[1].z2 + 5*module->states[1].z3;
}
