/*
 *	@title		Command Line Interface functions, user interface
 *	@context  	BoomBox & MMC-Box
 *  @author		O.C.Spro <ocspro@gmail.com>
 *	@file		cli_user_functions.c
 */

#include <string.h>
#include <stdio.h>
#include <math.h>

#include "DSP28x_Project.h"						// Headers for C28346
#include "../Core/macros.h"						// Global defines and macros
#include "../Driver/CLI/cli.h"					// Command line interface interpreter
#include "../inc/extern_user.h"					// Access global variables in user.c
#include "config.h"								// Project configuration
#include "SOGI3I.h"								// SOGI with triple integrator approximation
#include "../support_functions.h"


//CLOSED LOOP FUNCTIONS
void setReference();							//set reference point for closed loop current control

//OPEN LOOP FUNCTIONS
void setDutyCycle();							//set duty cycle for open loop control

//DEVICE FUNCTIONS
void deviceTurnOn();							//Initialize variables to default value
void deviceTurnOff();							//Kill the current safely and close all switches

// DEBUGGING AND MANUAL CONTROL
void debug();									//enter debug mode
void togglePrint();								//toggle printing of sensor values to screen on and off
void switchOn();								//in debug mode, turn optic output on
void switchOff();								//in debug mode, turn optic output off
void switchD();									//in debug mode, set duty cycle for a given switch
void SOGI_cycle();								//run one cycle of the SOGI module

//CHANGING PARAMETERS OF MODULES
void setSOGI_gain();							//set the gain of the SOGI 3I module
void setSOGI_const();							//set the constant of the SOGI 3I module
void setFilterParam();							//set the constant of the LPF module
void setPLLParam();								//set the two gains of the PLL module

// SUPPORT FUNCTIONS/PROCESSING INPUT STRINGS FROM TERMINAL
char switchnumToChar(char *input);
int inputCheck(char *input);
int charToNumber(char mychar);
int charToNumber_p(char *mychar);
int inputToInt(char *input);


void LoadCLIUserFunctions(void)
{
	/* This function is called by the core at system startup.
	 * the user-defined functions have to be registered
	 * by the command line interpreter before they can be used.
	 *
	 * These functions will be mapped in the /user folder in
	 * the command line mode. Use 'user' in order to be in this
	 * folder before use commands.
	 */
	fs_mkcmd_user("setRef", setReference);

	fs_mkcmd_user("setDC", setDutyCycle);

	fs_mkcmd_user("on", deviceTurnOn);
	fs_mkcmd_user("off", deviceTurnOff);

	fs_mkcmd_user("debug", debug);
	fs_mkcmd_user("print", togglePrint);
	fs_mkcmd_user("swOn", switchOn);
	fs_mkcmd_user("swOff", switchOff);
	fs_mkcmd_user("swD", switchD);
	fs_mkcmd_user("step", SOGI_cycle);
	fs_mkcmd_user("gain", setSOGI_gain);
	fs_mkcmd_user("const", setSOGI_const);
	fs_mkcmd_user("filt", setFilterParam);
	fs_mkcmd_user("PLL", setPLLParam);
}

/*
 * void setRef(int current)
 * set reference point for closed loop current control
 */
void setReference(unsigned int argc, char *argv[])
{
	if(user_state == WAITING || user_state == CLOSED_LOOP_CONTROL)
	{
		if(argc == 2)
		{
			if(!flagInit)
			{
				deviceTurnOn();
				initRegulator(&current_reg);
			}

			PWM_d.bias = 0;
			PWM_d.amplitude = 0;
			PWM_d.frequency = 0;
			PWM_d.phase_shift = 0;

			currentReference = (float)inputToInt(argv[1]) / 1000;
		}
	}
}

/*
 * void setDC(int bias, optional int amplitude, optional int frequency, optional int phase_shift)
 * set duty cycle for open loop control
 * bias = input / 1000
 * amplitude = input / 1000
 * frequency = input, in Hz
 * phase shift = input / 360 * PI (input whole degrees, is recalculated to radians)
 */
void setDutyCycle(unsigned int argc, char *argv[])
{
	if(user_state == WAITING || user_state == OPEN_LOOP_CONTROL)
	{
		if(!flagInit) deviceTurnOn();

		//set default PWM values
		PWM_d.bias = 0;
		PWM_d.amplitude = 0;
		PWM_d.frequency = 0;
		PWM_d.phase_shift = 0;



		//break command omitted on purpose between cases
		switch (argc)
		{
			case 5:{
				PWM_d.phase_shift = (double)inputToInt(argv[4]) / 360 * PI;
			}
			case 4:{
				PWM_d.amplitude = (float)inputToInt(argv[2]) / 1000;
				PWM_d.frequency = inputToInt(argv[3]);
			}
			case 2:{
				PWM_d.bias = (float)inputToInt(argv[1]) / 1000;
				user_state = OPEN_LOOP_CONTROL;
				break;
			}
			default:{
				printf("\nState machine is not ready");
			}
		}
		//printf("\nOPEN: argc:%d,b:%e,a:%e,f:%d,p:%e",argc,PWM_d.bias, PWM_d.amplitude,PWM_d.frequency,PWM_d.phase_shift);
	}
}

/*
 *	Turn on basic functionality of the AC Chopper to prepare it for use.
 *	Initiates SOGI
 */
void deviceTurnOn()
{
	//SOGI 3I parameters
	SOGI_init(&sogi);
	SOGI_init(&sogi_c);

	//PLL with standard parameters
	pll.DomegaPLL = 0.0;
	pll.IMprev  = 0.0;
	pll.omegaPLLprev  = 0.0;
	pll.THETA  = 0.0;
	pll.Ts = T_SAMPLE;
	pll.omega = SOGI_OMEGA;
	pll.aB = PLL_A;
	pll.kB = PLL_K;

	//Filter paramters

	//set default PWM values
	PWM_d.bias = 0;
	PWM_d.amplitude = 0;
	PWM_d.frequency = 0;
	PWM_d.phase_shift = 0;

	//set flag values
	flagInit = 1;

	switch_state = START;

	printf("\nAC Chopper initialized");
}

/*
 *	Safely turn the AC Chopper off. Any remaining inductor current is set to be dispersed
 *	internally in the free wheeling loop. When the current is zero all the switches are
 *	turned off
 */
void deviceTurnOff()
{
	//set free wheeling mode
	setSwitchFW();

	user_state = CLEAN_UP;

	//set flag values
	flagInit = 0;
}

/*
 *	Run the SOGI 3I for one cycle by terminal input
 *	Voltage reading to LEM_voltage must be updated before calling the method
 */
void SOGI_cycle()
{
	SOGI_step(&sogi, LEM_voltage);
}

/*
 * void setSOGI_gain(int gain_3I, int gain_built_in)
 * Set the gain of the two SOGI modules
 * both the gains are set: gain = input / 100
 */
void setSOGI_gain(unsigned int argc, char *argv[])
{
	//set gain of the SOGI 3I module
	sogi.gain = (float)inputToInt(argv[1])/100;
}

/*
 *	void setSOGI_const(int frequency)
 *	set the internal SOGI 3I constant
 *	constant = Ts / 12 = 1 / 12f_s , where Ts is the sampling time, and f_s is the sampling frequency
 *	input frequency in Hz
 */
void setSOGI_const(unsigned int argc, char *argv[])
{
	sogi.constant = (1 / ((float)inputToInt(argv[1]) * 12));
}

/*
 *	void setFilterParam(int constant)
 *	set the low pass filter constant
 *	constant = 1 / input
 */
void setFilterParam(unsigned int argc, char *argv[])
{
	//filter.FilterConstant = 1 / ((float)inputToInt(argv[1]));
	double answ = sin((double)inputToInt(argv[1])/100.0);
	double answ2 = sin((double)inputToInt(argv[1])/100.0 + inputToInt(argv[2]));
	float answ3 = answ + 1.0;
	printf("\nTEST: in:%d,sin:%e,sin2:%e,float:%e",inputToInt(argv[1]),answ, answ2, answ3);
}

/*
 * void setPLLParam(int gain1, int gain2)
 * set the two gains, aB and kB for the PLL module
 * gain = input / 10
 */
void setPLLParam(unsigned int argc, char *argv[])
{
	//reset PLL parameters and add new gain
	pll.DomegaPLL = 0.0;
	pll.IMprev  = 0.0;
	pll.omegaPLLprev  = 0.0;
	pll.THETA  = 0.0;
	pll.Ts = T_SAMPLE;
	pll.omega = SOGI_OMEGA;
	pll.aB = (float32)inputToInt(argv[1]) / 10.0;
	pll.kB = (float32)inputToInt(argv[2]) / 10.0;

	flagFillBuffer = 1; //update debugging buffer
}

/*
 * void debug(void)
 * State machine enters the debug mode, allowing setting of individual switches and
 * allows printout of more device information to the terminal.
 */
void debug(unsigned int argc, char *argv[])
{
	if(user_state == WAITING)
	{
		user_state = DEBUG;
		printf("\nRunning DEBUG mode");
	}
	else if(user_state == DEBUG)
	{
		deviceTurnOff();
		printf("\nExiting DEBUG mode");
	}
	else
	{
		printf("\nState machine is not ready");
	}
}

/*
 *	void togglePrint(void)
 *	Toggles printing on and off in debug mode, prints sensor values to terminal every ~3 second
 */
void togglePrint(unsigned int argc, char *argv[])
{
	if (user_state == DEBUG)
	{
		if(flagPrint) flagPrint = 0;
		else flagPrint = 1;
	}
}


/*
 * void switchOn(int switch)
 * in debug mode, turn optic output on.
 * switch = [1,4]
 */
void switchOn(unsigned int argc, char *argv[])
{
	if (user_state == DEBUG)
	{
		if(argc == 2)
		{
			if (inputCheck(argv[1])) setPWMDutyCycle(MULTIA_SLOT,switchnumToChar(argv[1]),1);
		}
	}
}

/*
 * void switchOff(int switch)
 * in debug mode, turn optic output off
 * switch = [1,4]
 */
void switchOff(unsigned int argc, char *argv[])
{
	if (user_state == DEBUG)
	{
		if(argc == 2)
		{
			if (inputCheck(argv[1])) resetPWMDutyCyclePair(MULTIA_SLOT,switchnumToChar(argv[1]));
		}
	}
}

/*
 * void switchOn(int switch, int duty)
 * in debug mode, set duty cycle for a given switch
 * switch = [1,4], duty = [0,1000]
 */
void switchD(unsigned int argc, char *argv[])
{
	if (user_state == DEBUG)
	{
		if(argc == 3)
		{
			if (inputCheck(argv[1])) setPWMDutyCycle(MULTIA_SLOT, switchnumToChar(argv[1]), (float)inputToInt(argv[2])/1000);
		}
	}
}

/*
 * 	Translates number in char format to corresponding capital letter.
 * 	Used to translate input number to corresponding switch.
 * 	Examples:
 * 	INPUT:
 * 	char c = '1';
 *	switchnumToChar(&c);
 * 	OUTPUT: 'A'
 * 	INPUT:
 * 	char c = '3';
 *	switchnumToChar(&c);
 * 	OUTPUT: 'C'
 */
char switchnumToChar(char *input)
{
	return((char)(charToNumber_p(input) - 1) + 'A');
}

/*
 * 	Checks if input switch number is valid, i.e. [1,4] and returns the switch number
 * 	INPUT:
 * 	char c = '1';
 * 	inputCheck(&c);
 * 	OUTPUT: 1
 * 	INPUT:
 * 	char c = '5';
 * 	inputCheck(&c);
 * 	OUTPUT: 0
 */
int inputCheck(char *input)
{
	int i = charToNumber_p(input);
	if (i < 0 || i > 4)
	{
		printf("\nSwitch number must be 1,2,3 or 4");
		i = 0;
	}
	return (i);
}

/*
 * 	Translates a char to corresponding int
 * 	INPUT: charToNumber('4');
 * 	OUTPUT: 4
 */
int charToNumber(char mychar)
{
	return((int)(mychar - '0'));
}

/*
 * 	Translates char in the pointer to corresponding int
 * 	INPUT:
 * 	char c = '4';
 * 	charToNumber(&c);
 * 	OUTPUT: 4
 */
int charToNumber_p(char *mychar)
{
	return((int)(*mychar - '0'));
}

/*
 * 	Translates a number in string format in the pointer to corresponding int
 * 	INPUT:
 * 	char c = '451';
 * 	inputToInt(&c);
 * 	OUTPUT: 451
 */
int inputToInt(char *input)
{
	int number = 0, i;
	for(i=0;i<strlen(input);i++)
	{
		number = number * 10;
		number += charToNumber(input[i]);
	}
	return number;
}
