#ifndef CORE_H_
#define CORE_H_


/**
 * Enum type for the state machine
 */
typedef enum
{
	ERROR  = 0,			// The box got an error signal and wait for an acknowledgment signal to go back to the init state
	IDLE   = 1,			// The box is configured and wait for a start signal to go to the starting state
	NORMAL = 2			// The box is running properly and wait for a stop signal to go to the stopping state
} CORE_STATE;


/**
 * Informs the user about "who" asked to enter in error mode
 * This enum is intended to be passed when calling UserError();
 */
typedef enum
{
	HARD = 0,			// Signal FAULT from PERIPH
	SOFT = 1,			// Error from the core software
	USER = 2			// User error from his software
} ERROR_SOURCE;

/**
 * Configures the PIE vector in order that each interrupt
 * maps to the function UnmappedInterruptTrap. Thus inexpected interruptions
 * are catchable and are easy to debug
 */
void SetPieVectTable(void);


/**
 * Does the initialization of the core.
 * The peripherals, watchdog and I/Os are configured in this function. This function
 * is also calling the UserInit() function.
 */
void CoreInit(void);


/**
 * Do all things the core has to do to start the experiment (start interrupts, open outputs,...)
 * This function calls UserStart().
 * @return void
 */
void CoreStart(void);


/**
 * Do all things the core has to do to stop the experiment (stop interrupts, close outputs,...)
 * This function calls UserStop().
 * @return void
 */
void CoreStop(void);


/**
 * Do all things the core has to do in background.
 * All tasks not time dependent are done inside this function.
 * The watchdog counter is also reset.
 * @return void
 */
void CoreBackground(void);


/**
 * This function is the ISR serving the most critical hardware (applicative) errors
 * Its purpose is to prepare proper CPU reboot. This function calls UserError().
 */
void CoreError(ERROR_SOURCE source);

/**
 * This function notifies the user that the device requires to be rebooted.
 * Locks the device in a loop and reset it when user press a key (console!)
 */
void CoreForceReboot(void);

#endif /*CORE_H_*/
