#ifndef INTERRUPTS_H_
#define INTERRUPTS_H_

#include "../Driver/irq.h"						// FPGA-related register definitions
#include "../extern_user.h"						// Pre-definition of user functions


/**
 * Configures the PIE vector in order that each interrupt
 * maps to the function UnmappedInterruptTrap. Thus inexpected interruptions
 * are catchable and are easy to debug
 * @param void
 * @return void
 */
void SetPieVectTable(void);


/**
 * This routine is responsible for configuring the XINT1 interrupt belonging to the PIE1 group
 * @param source the peripheral that should be used to generate the IRQ
 * @param edge the kind of edge that triggers the interrupt (0 for falling, 1 for rising)
 * @return void
 */
void ConfigExt1Interrupt(IRQ_SOURCES source, Uint16 edge);


/**
 * This routine is responsible for configuring the XINT1 interrupt belonging to the PIE1 group
 * @param source the peripheral that should be used to generate the IRQ
 * @param edge the kind of edge that triggers the interrupt (0 for falling, 1 for rising)
 * @return void
 */
void ConfigExt2Interrupt(IRQ_SOURCES source, Uint16 edge);


/**
 * This routine is responsible for configuring the Timer interrupt belonging to the PIE1 group
 * @param period the period of the CPU Timer
 * @return void
 */
void ConfigTimerInterrupt(Uint16 period);


/**
 * This routine is used to register a user-level ISR (callback function) to the CoreInterrupt1 ISR
 * The user is free to register any function as "top-level" ISR
 * @param source the peripheral that should be used to generate the IRQ
 * @param edge the kind of edge that triggers the interrupt (0 for falling, 1 for rising)
 * @return void
 */
void RegisterExt1Interrupt(USER_SAFE (*user_isr)(void),IRQ_SOURCES source, Uint16 edge);


/**
 * This routine is used to register a user-level ISR (callback function) to the CoreInterrupt2 ISR
 * The user is free to register any function as "top-level" ISR
 * @param source the peripheral that should be used to generate the IRQ
 * @param edge the kind of edge that triggers the interrupt (0 for falling, 1 for rising)
 * @return void
 */
void RegisterExt2Interrupt(USER_SAFE (*user_isr)(void),IRQ_SOURCES source, Uint16 edge);


/**
 * This routine is used to register a user-level ISR (callback function) to the CPU Timer 0
 * The user is free to register any function as "top-level" ISR
 * @param *user_isr pointer to the user interrupt that will be served
 * @param period the period of the CPU Timer between 1us and 65535us (65ms)
 * @return void
 */
void RegisterTimerInterrupt(USER_SAFE (*user_isr)(void), Uint16 period);


/**
 * This routine is responsible for configuring the non-maskable interrupt service-routine
 * It handles the most critical hardware errors triggered by the PWM_nFAULT signals
 * @param edge the kind of edge that triggers the interrupt (0 for falling, 1 for rising)
 * @return void
 */
void ConfigXNMIInterrupt(int16 edge);


/**
 * ISR routine for the Xternal hardware interrupt 1
 * The user is free to register any callback function on this ISR
 * @see RegisterExt1Interrupt()
 * @param void
 * @return void
 */
interrupt void Ext1Interrupt(void);


/**
 * ISR routine for the CPU Timer interrupt
 * The user is free to register any callback function on this ISR
 * @see RegisterTimerInterrupt()
 * @param void
 * @return void
 */
interrupt void TimerInterrupt(void);


/**
 * ISR routine for the Xternal hardware interrupt 2
 * The user is free to register any callback function on this ISR
 * @see RegisterExt2Interrupt()
 * @param void
 * @return void
 */
interrupt void Ext2Interrupt(void);

/**
 * Stops the converter. Warns the user thar program has reached an unexpected error.
 * Displays the state of all IFRs and IERs and the 10 last elements of SP.
 * @param void
 * @return void
 */
interrupt void UnmappedInterruptTrap(void);

/**
 * ISR routine for the hardware error interrupt (NMI)
 * The routine calls the CoreError function with an MATERIAL code error
 * @see ConfigXNMIInterrupt()
 * @param void
 * @return void
 */
interrupt void XNMIInterrupt(void);


/**
 * Function that checks if the user software is going good !
 * @return return 0 if OK, 1 else
 */
int CheckInterruptStatus(void);


/**
 * Displays the state of all IFRs and IERs and the 10 last elements of SP.
 * @param void
 * @return void
 */
interrupt void UnmappedInterruptTrap(void);


#endif /* INTERRUPTS_H_ */
