#ifndef MAPPING_H_
#define MAPPING_H_

#include "DSP28x_Project.h"		// Headers for C28346

/* 
 * This file defines the hardware/software mappings.
 * These definitions are common to the BoomBox and the MMC-Box
 */


struct SUP_BBOX {
	int16 STATE;					// Intended for the main state machine
	int16 rsvd0[255];
};

struct ADC_BBOX {
	int16 DATA[6];					// Six channels per ADC
	int16 rsvd0b[58];
};

struct PWM_BBOX {
	Uint16 ARESET;
	Uint16 PRESCALER;
	Uint16 PERIOD;					// Max period of the PWM counter
	Uint16 RISHA;					// Rising edge of the high-channel of pair A
	Uint16 FALHA;					// Falling edge of the high-channel of pair A
	Uint16 RISLA;					// Rising edge of the low-channel of pair A
	Uint16 FALLA;					// Falling edge of the low-channel of pair A
	Uint16 RISHB;
	Uint16 FALHB;
	Uint16 RISLB;
	Uint16 FALLB;
	Uint16 RISHC;
	Uint16 FALHC;
	Uint16 RISLC;
	Uint16 FALLC;
	Uint16 RISHD;
	Uint16 FALHD;
	Uint16 RISLD;
	Uint16 FALLD;
	int16 rsvd1[13];
};

struct GPO_BBOX {
	Uint16 DATA;					// Only the last 8 bits are active
	int16 rsvd2[255];
};

struct GPI_BBOX {
	Uint16 DATA;					// Only the last 8 bits are active
	int16 rsvd3[255];
};

struct SDR_BBOX {
	int16 rsvd4[256];
};

struct CLK_BBOX {
	Uint16 ARESET;					// Only the 8 last bits are active
	Uint16 HALFPERIOD;				// Only the 8 last bits are active
	int16 fill_vector[254];
};

struct IRQ_BBOX {
	Uint16 SOURCE1;
	Uint16 SOURCE2;
	int16 fill_vector[254];
};

struct SUP_MEAS {
	int16 ERR_ARST;					// Intended for the main state machine
	int16 rsvd5[255];
};

struct SDR_MEAS {
	int16 rsvd6[256];
};

struct GPO_MEAS {
	Uint16 DATA;
	int16 rsvd7[255];
};

struct TEST_RAM {
	Uint16 DATA[16];
	int16 rsvd7b[240];
};

struct WD_BBOX {
	Uint16 enable;
	Uint16 period;
	int16 rsvd7b[254];
};


struct ACQ_MEAS {
	Uint16 STATE;
	Uint16 OVERVALUE;				// Contains flags that are activated when positive (upper)
									// limits are reached (bit rank is channel number)
	Uint16 UNDERVALUE;				// Contains flags that are activated when negative (lower)
									// limits are reached (bit rank is channel number)
	Uint16 rsvd8[13];
	int16 DATA[12];					// Data registers (12 channels)
	Uint16 rsvd9[4];
	int16 LIMP[12];					// Positive (upper) limits registers
	Uint16 rsvd10[4];
	int16 LIMN[12];					// Negative (lower) limits registers
	int16 rsvd11[196];
};


struct  CONFIG_BITS {        		// bits   	description
   Uint16 ENABLE:1;           		// 0		enable converter operation
   Uint16 FB:1;            			// 1      	enable full-bridge operation (0 for half-bridge)
   Uint16 AnB:1;            		// 2      	enable full-bridge A or full bridge B
   Uint16 rsvd99:13;           		// 15:3   	reserved
};

union CONFIG_REG {
   Uint16              all;
   struct CONFIG_BITS  bit;
};

struct SUP_BRANCH {
	union CONFIG_REG CONFIG;		// Configuration register
	Uint16 CLRFLAG;					// Clear error flags (async reset)
	int16 rsvd12[254];
};

struct ACQ_BRANCH {
	Uint16 OVERVALUE0;				// SPI(0-15) (bit rank is channel number)
	Uint16 OVERVALUE1;				// SPI(16-32) (bit rank is channel number)
	Uint16 UNDERVALUE0;				// SPI(0-15) (bit rank is channel number)
	Uint16 UNDERVALUE1;				// SPI(16-32) (bit rank is channel number)
	Uint16 rsvd8[28];

	Uint16 DATA[32];				// All data channels are considered to be unsigned (event the arm currents...)
	Uint16 LIMP[32];
	Uint16 LIMN[32];
	int16 rsvd13[128];
};

struct ARM_BRANCH {
	int16 ACTIVE_CELLS;
	Uint16 DUTY_CYCLE;
	Uint16 LIMIT;
	Uint16 DEADTIME;
	Uint16 ENABLEDCELLS;
	int16 rsvd14[251];
};


/*
 * XINTF Peripherals :
 * These are the physical peripherals that can be addressed by the CPU
 */

struct FPGA_BBOX {
	struct SUP_BBOX sup;			// Supervisory unit
	struct ADC_BBOX adc[4];			// 4 modules of 6 channels are implemented in FPGA
	struct PWM_BBOX pwm[4];			// 4 modules of 4 double-channels are implemented in FPGA
	int16 rsvd15[256-4*32];
	struct SDR_BBOX sdr;			// Synchronous Data Ram block
	struct GPO_BBOX gpo;			// General-purpose 8 bit output register
	struct GPI_BBOX gpi;			// General-purpose 8 bit input register
	struct CLK_BBOX clk;			// Clock generation module (ACQ)
	struct IRQ_BBOX irq;			// Interrupt request generator block
	struct TEST_RAM testram;
	struct WD_BBOX wd;				// FPGA watchdog
	int16 rsvd16[4096-10*256];
};

struct FPGA_BRANCH {
	struct SUP_BRANCH sup;			// Supervisory unit
	struct CLK_BBOX clk;			// Clock generation module (SPI)
	struct ACQ_BRANCH acq;			// 2x16 muxed SPI channels with two limit registers
	struct ARM_BRANCH arm[2];		// Arm controllers (0 is "up", 1 is "low")
	struct PWM_BBOX pwm[2];			// 2 modules of 8 channels are implemented in FPGA
	int16 rsvd17[256-2*32];
	int16 rsvd18[4096-6*256];
};

struct FPGA_MEAS {
    struct SUP_MEAS sup;			// Supervisory unit
	struct ACQ_MEAS acq;			// Acquisition block
	struct SDR_MEAS sdr;			// Synchronous Data Ram block
	struct GPO_MEAS gpo;			// General-purpose output block
	int16 rsvd19[4096-4*256];
};

struct XINTF6D {
	struct FPGA_BBOX bbox;			// BoomBox
	struct FPGA_MEAS extio[2];		// MMCBox
	struct FPGA_BRANCH branch[12];	// MMCBox
};


extern volatile struct XINTF6D XD6;

// Alias old variable name to ensure code retro-compatibility
#define FPGA XD6.bbox


/*
 * Parameters for the FPGA implementation of logical peripherals
 * Some parameters might differ in the MMCBox implementation
 */
#define QTY_OF_PWM_MODULES 4
#define QTY_OF_PWM_CHANNELS 4
#define QTY_OF_ADC_CHANNELS 6


#endif /*MAPPING_H_*/
