/*
 *  @title    Functions to implement open and closed loop control of an AC Chopper
 * 	@context  BoomBox & MMC-Box
 *	@author   O.C. Spro <ocspro@gmail.com>
 *	@file     support_functions.h
 *	@comment  last updated: 22.07.2013
*/

#include <math.h>
#include "My_functions/config.h"
#include "inc/extern_user.h"
#include "support_functions.h"


void switchingControl()
{
	float duty;

	switch (switch_state){
		case COMMUTATION_POS: {
			if(voltage_diff < 0 && (LEM_voltage < VOLTAGE_FALLING_THRESHOLD && LEM_voltage > 0))
			{
				setSwitchFW();
				next_switch_state = FREE_WHEEL;
			}else {
				duty = calculateDutycycle();
				setPWMDutyCycle(MULTIA_SLOT,'A',duty);
				setPWMDutyCycle(MULTIA_SLOT,'B',duty);
				next_switch_state = COMMUTATION_POS;
			}
			break;
		}
		case COMMUTATION_NEG: {
			if(voltage_diff > 0 &&(LEM_voltage >-VOLTAGE_FALLING_THRESHOLD && LEM_voltage < 0))
			{
				setSwitchFW();
				next_switch_state = FREE_WHEEL;
			}else {
				duty = calculateDutycycle();
				setPWMDutyCycle(MULTIA_SLOT,'C',duty);
				setPWMDutyCycle(MULTIA_SLOT,'D',duty);
				next_switch_state = COMMUTATION_NEG;
			}
			break;
		}
		case FREE_WHEEL: {
			if(checkVoltageOverThreshold())
			{
				if(checkVoltagePolarity())
				{
					setSwitchComPos();
					next_switch_state = COMMUTATION_POS;
				}else {
					setSwitchComNeg();
					next_switch_state = COMMUTATION_NEG;
				}
			}else {
				//if(checkZeroCurrent()) setSwitchOff();
				next_switch_state = FREE_WHEEL;
			}
			break;
		}
		case START: {
			if(LEM_voltage > 15) next_switch_state = COMMUTATION_POS;
			else if (LEM_voltage < 15) next_switch_state = COMMUTATION_NEG;
			else next_switch_state = START;
			break;
		}
	}

	switch_state = next_switch_state;
}


float runRegulator(RegulatorParameters *reg, float error)
{
	/*
	 * Expression to implement:
	 * OUT(k) = OUT(k-1) + a1*IN(k) + a2*IN(k-1) + a3*IN(k-2) + a4*IN(k-3) + a5*IN(k-4)
	 */
	reg->new_output = reg->old_output + reg->parameters[0]*error;
	reg->new_output += reg->parameters[1]*reg->old_input[0];
	reg->new_output += reg->parameters[2]*reg->old_input[1];
	reg->new_output += reg->parameters[3]*reg->old_input[2];
	reg->new_output += reg->parameters[4]*reg->old_input[3];

	/*
	//limit upper or lower value
	if(reg->new_output > REG_UPPER) reg->new_output = REG_UPPER;
	if(reg->new_output < REG_LOWER) reg->new_output = REG_LOWER;
	*/

	// update old/new values
	reg->old_output = reg->new_output;
	reg->old_input[3] = reg->old_input[2];
	reg->old_input[2] = reg->old_input[1];
	reg->old_input[1] = reg->old_input[0];
	reg->old_input[0] = error;

	return reg->new_output;
}

void initRegulator(RegulatorParameters *reg)
{
	reg->old_output = 0;
	reg->new_output = 0;
	reg->old_input[0] = 0;
	reg->old_input[1] = 0;
	reg->old_input[2] = 0;
	reg->old_input[3] = 0;
	reg->parameters[0] = REG_A1;
	reg->parameters[1] = REG_A2;
	reg->parameters[2] = REG_A3;
	reg->parameters[3] = REG_A4;
	reg->parameters[4] = REG_A5;
}

/*
 * Switch setup for commutation with positive input voltage
 */
void setSwitchComPos()
{
	setPWMDutyCycle(MULTIA_SLOT,'A',calculateDutycycle());
	setPWMDutyCycle(MULTIA_SLOT,'B',calculateDutycycle());
	setPWMDutyCycle(MULTIA_SLOT,'C', 1);
	setPWMDutyCycle(MULTIA_SLOT,'D', 0);
}

/*
 * Switch setup for commutation with negative input voltage
 */
void setSwitchComNeg()
{
	setPWMDutyCycle(MULTIA_SLOT,'A', 1);
	setPWMDutyCycle(MULTIA_SLOT,'B', 0);
	setPWMDutyCycle(MULTIA_SLOT,'C',calculateDutycycle());
	setPWMDutyCycle(MULTIA_SLOT,'D',calculateDutycycle());
}

/*
 * Switch setup for free wheeling path
 */
void setSwitchFW()
{
	resetPWMDutyCyclePair(MULTIA_SLOT,'A');
	setPWMDutyCycle(MULTIA_SLOT,'B',0);
	resetPWMDutyCyclePair(MULTIA_SLOT,'C');
	setPWMDutyCycle(MULTIA_SLOT,'D',0);

}


/*
 * Switch setup for all switches off
 */
void setSwitchOff()
{
	resetPWMDutyCyclePair(MULTIA_SLOT,'A');
	resetPWMDutyCyclePair(MULTIA_SLOT,'B');
	resetPWMDutyCyclePair(MULTIA_SLOT,'C');
	resetPWMDutyCyclePair(MULTIA_SLOT,'D');
}


void sampleSensors()
{
	static float old_voltage[2];
	old_voltage[1] = old_voltage[0];
	old_voltage[0] = LEM_voltage;
	LEM_voltage = getADC(0,0);
	LEM_current = getADC(0,1);
	voltage_diff = (LEM_voltage - old_voltage[1]);
}

/*
 * 	Returns 1 if positive or zero voltage is measured, else returns 0
 */
int checkVoltagePolarity()
{
	if (LEM_voltage >= 0) return 1;
	else return 0;
}

/*
 * 	Returns 1 if positive or zero current is measured, else returns 0
 *//*
int checkCurrentDirection()
{
	if (LEM_current >= 0) return 1;
	else return 0;
}*/

/*
 * 	Checks if grid voltage has risen above the set threshold.
 * 	Used in free wheeling switching states
 */
int checkVoltageOverThreshold()
{
	if((LEM_voltage > VOLTAGE_RISING_THRESHOLD && voltage_diff > 0)|| (LEM_voltage < -VOLTAGE_RISING_THRESHOLD && voltage_diff < 0)) return 1;
	else return 0;
}

/*
 * 	Checks if grid voltage has fallen below the set threshold.
 * 	Used in commutation states
 */
int checkVoltageUnderThreshold()
{
	if(LEM_voltage < VOLTAGE_FALLING_THRESHOLD && LEM_voltage > -VOLTAGE_FALLING_THRESHOLD) return 1;
	else return 0;
}

/*
 *  Returns 1 if current zero (I<0.01A), else returns 0
 */
int checkZeroCurrent()
{
	if(LEM_current < CURRENT_THRESHOLD && LEM_current > -CURRENT_THRESHOLD) return 1;
	else return 0;
}

/*
 * 	Calculate open loop duty cycle, relative to grid voltage theta.
 * 	Requires a running SOGI and PLL device
 */
float calculateDutycycle()
{
	float d = PWM_d.bias + PWM_d.amplitude * ((float)sin((double)(PWM_d.frequency/50.0 * pll.THETA) + PWM_d.phase_shift));

	if(d<PWM_MIN) return 0.0;
	else if (d>PWM_MAX) return 1.0;
	else return d;
}
