/*
 *  @title    User-level main file: TEMPLATE TO BE MODIFIED !
 * 	@context  BoomBox & MMC-Box
 *	@author   N.Cherix <nicolas.cherix@epfl.cha> & S. Delalay <simon.delalay@epfl.ch>
 *	@file     user_main.c
 *	@comment  Core functions require that this file implements these functions :
 *			  USER_SAFE UserInit(void)
 *			  USER_SAFE UserStart(void)
 *			  USER_SAFE UserStop(void)
 *			  USER_SAFE UserInterrupt1(void)
 *			  void UserError(int code);
 */

#include "DSP28x_Project.h"				// Headers for C28346
#include "Core/macros.h"				// Global defines and macros
#include "Driver/peripherals.h"			// Available DSP peripherals
#include "Core/interrupts.h"			// Low-level and user interrupts handling functions
#include "Core/core.h"					// Core functions
#include <stdio.h>
#include <math.h>

#include "inc/extern_user.h"			// This file header.
#include "My_functions/config.h"		// Project configuration
#include "My_functions/SOGI3I.h"		// Sogi module
#include "API/control_functions.h"		// functions used for sogi and pll modules
#include "support_functions.h"			// functions used for open and closed loop regulation

/**
 * Global variables
 * Please pay attention not to define variables that are unnecessarily global !
 */

//STATES FOR STATE MACHINES
USER_STATE user_state, next_user_state;
SWITCH_STATE switch_state, next_switch_state;


//FLAGS
int flagFillBuffer = 0;			//fill buffers with synchronized read out values to debug or print graphs/figures
int flagInit = 0;				//force initialization before running control schemes
int flagPrint = 0;				//print to terminal in DEBUG mode


//REG, LPF, SOGI and PLL variables
static SpacePhasor ab, dq;
PLLparameters pll;
SOGI_3I_module sogi, sogi_c;
RegulatorParameters current_reg;
float currentReference;
float altRMS;
float values[40];

//LEM VARIABLES
float LEM_voltage, LEM_current;
float voltage_diff;


//DEBUG BUFFER VARIABLES
float LEM_buffer_v[100], LEM_buffer_c[100];
float SOGI_v_3i[DEBUG_BUFFER_SIZE], SOGI_RMS_3i[DEBUG_BUFFER_SIZE], theta_3i[DEBUG_BUFFER_SIZE];
float duty[DEBUG_BUFFER_SIZE];
int state_buffer[100];

//DUTY CYCLE VARIABLES
DutycycleParameters PWM_d;



/**
 * The user should put here its initialization routines.
 * This function is called only once at startup.
 */
USER_SAFE UserInit(void){

	// Initialize all peripherals :
	InitAllADC();						// Initialize all AD Converters with their default corrective parameters (gain+offset)
	InitAllPWM();						// Initialize all PWM modules with their default values
	InitAllGPO();						// Initialize all FPGA-based GPOs

	// Configure peripherals :
	setADCAdjustments(MULTIA_SLOT,LEM_VOLTAGE_CHANNEL,LEM_VOLTAGE_GAIN,LEM_VOLTAGE_OFFSET);	// Configure input voltage sensor output
	setADCAdjustments(MULTIA_SLOT,LEM_CURRENT_CHANNEL,LEM_CURRENT_GAIN,LEM_CURRENT_OFFSET);	// Configure impedance current sensor output

	setPWMPrescaler(MULTIA_SLOT,PWM_PRESCALER);		//configure prescaler of the PWM
	setPWMPeriod(MULTIA_SLOT,PWM_PERIOD);			//configure period of the PWM
	setPWMDeadTime(MULTIA_SLOT,'A',PWM_DEADTIME);
	setPWMPhase(MULTIA_SLOT, 'A', PWM_PHASE);
	setPWMDeadTime(MULTIA_SLOT,'B',PWM_DEADTIME);
	setPWMPhase(MULTIA_SLOT, 'B', PWM_PHASE);
	setPWMDeadTime(MULTIA_SLOT,'C',PWM_DEADTIME);
	setPWMPhase(MULTIA_SLOT, 'C', PWM_PHASE);
	setPWMDeadTime(MULTIA_SLOT,'D',PWM_DEADTIME);
	setPWMPhase(MULTIA_SLOT, 'D', PWM_PHASE);

	// Configure user interrupts (examples) :
	//RegisterTimerInterrupt(&UserInterrupt1,100);
	RegisterExt1Interrupt(&UserInterrupt1,PWM0,RISING);

	// Initialize the application to its STANDBY state
	user_state = WAITING;
	int i;
	for(i=0;i<40;i++) values[i] = 0;

	return SAFE;									// Return 0 if no error
}

/**
 * The user should put here its starting routines.
 * This function is called by CoreStart().
 */
USER_SAFE UserStart(void)
{
	return SAFE;							// Return 0 if no error
}

/**
 * The user should put here its stopping routines.
 * This function is called by CoreStop().
 */
USER_SAFE UserStop(void)
{
	return SAFE;							// Return 0 if no error
}

/**
 * This is the interrupt with the highest priority available to the user.
 * It should contain the main control routines.
 */
USER_SAFE UserInterrupt1(void)
{
	XD6.bbox.wd.enable = 0;							//DEBUG  disables FPGA checking for DSP pause/hangup or not

	// User's Finite State Machine
	switch (user_state){
	   	case WAITING:{
	   		sampleSensors();
	   		next_user_state = WAITING;
	   		break;
	   	}
	   	case OPEN_LOOP_CONTROL:{
	   		static int jx = 0;


	   		//2kHz
	   		sampleSensors();

	   		//calculate SOGI and PLL values

	   		SOGI_step(&sogi, LEM_voltage);
	   		SOGI_step(&sogi_c, LEM_current);
	   		ab.real = sogi.states[0].output;
	   		ab.imaginary = sogi.states[1].output;
	   		AlphaBeta2DQ(&dq,&ab,pll.THETA);
	   		PLL(&dq, &pll);


	   		//RMS discrete implementation
	   		/*
	   		int i;
	   		for(i=40;i>0;i--) values[i] = values[i-1];
	   		values[0] = LEM_voltage;
	   		altRMS = 0;
	   		for(i=0;i<40;i++) altRMS += pow(values[i],2);
	   		altRMS = sqrt(altRMS / 40);
			*/

	   		switchingControl();


	   		if(flagFillBuffer)	//DEBUG or EXPORT DATA OPTION
	   		{
	   			fillOutputBuffers(jx);

	   			if (jx++ == DEBUG_BUFFER_SIZE)
	   			{
	   				flagFillBuffer = 0;
	   				jx = 0;
	   			}
	   		}

	   		next_user_state = OPEN_LOOP_CONTROL;
	   		break;
	   	}
	   	case CLOSED_LOOP_CONTROL:{
	   		static int counter_500Hz = 0;
	   		static int jx = 0;
	   		float current_RMS;

	   		//2kHz
	   		sampleSensors();

	   		//calculate SOGI and PLL values
	   		SOGI_step(&sogi, LEM_current);
	   		ab.real = sogi.states[0].output;
	   		ab.imaginary = sogi.states[1].output;
	   		current_RMS = ONEOVERSQRT2 * sqrt(pow(ab.real,2) + pow(ab.imaginary,2));

	   		if(flagFillBuffer)	//DEBUG or EXPORT DATA OPTION
	   		{
	   			fillOutputBuffers(jx);

	   			if (jx++ == DEBUG_BUFFER_SIZE)
	   			{
	   				flagFillBuffer = 0;
	   				jx = 0;
	   			}
	   		}

	   		if(counter_500Hz == 4)
	   		{
	   			counter_500Hz = 0;

	   			//regulator
	   			PWM_d.bias = runRegulator(&current_reg, currentReference - current_RMS);
	   			switchingControl();
	   		}

	   		next_user_state = CLOSED_LOOP_CONTROL;
	   		break;
	   	}
	   	case DEBUG:{
	   		static int i = 0;

	   		if(i++ > 10000)
	   		{
		   		LEM_voltage = getADC(0,0);
		   		LEM_current = getADC(0,1);
		   		if(flagPrint) printf("\nVoltage: %e\nCurrent: %e", LEM_voltage, LEM_current);
		   		i = 0;
	   		}

	   		next_user_state = DEBUG;
	   		break;
	   	}
	   	case CLEAN_UP:{

	   		LEM_current = getADC(0,1);

	   		if (checkZeroCurrent())
			{
				//Current has died out, turn off all switches
	   			setSwitchOff();
				next_user_state = WAITING;
			}else {
				//wait for current to die out
				next_user_state = CLEAN_UP;
			}
	   		break;
	   	}
	   	case FAULT:{
	   		// Some code here...
	   		next_user_state = FAULT;
		   	break;
	   	}
	}

	// Switch to the next state
	user_state = next_user_state;
    
    return SAFE;						// Return a SAFE status (or anything else...)
}


/*
 * This routine aims to handle critical errors by requesting an appropriate answer at the user level.
 * Typically, the application should be blocked and the core will wait for manual clearing of the error.
 */
void UserError(int code){
	decodeError(code);
}

void fillOutputBuffers(int index)
{
	LEM_buffer_v[index] = LEM_voltage;
	LEM_buffer_c[index] = LEM_current;


	theta_3i[index] = pll.THETA;
	SOGI_RMS_3i[index] = sqrt(pow(sogi_c.states[0].output,2) + pow(sogi_c.states[1].output,2));
	SOGI_v_3i[index] = altRMS;

	duty[index] = calculateDutycycle();
	state_buffer[index] = switch_state;
}

void decodeError(int code)
{
	switch(code){
		case 1:{
			printf("\nError1");
			break;
		}
		case 2:{
			printf("\nError2");
			break;
		}
	}
}
